/*
	Copyright (c) 1997 Eugene G. Crosser

	You may do virtually what you wish with this software, as long
	as the explicit reference to its original author is retained.

	THIS SOFTWARE IS PROVIDED AS IS AND COME WITH NO WARRANTY OF ANY
	KIND, EITHER EXPRESSED OR IMPLIED.  IN NO EVENT WILL THE
	COPYRIGHT HOLDER BE LIABLE FOR ANY DAMAGES RESULTING FROM THE
	USE OF THIS SOFTWARE.
*/

#include "eph_io.h"
#include "eph_priv.h"
#include <stdio.h>

#define MAYRETRY(rc) ((rc == -2) || (rc == NAK))

int eph_writecmd(eph_iob *iob,char *data,size_t length) {
	return eph_writepkt(iob,PKT_CMD,SEQ_CMD,data,length);
}

int eph_writeicmd(eph_iob *iob,char *data,size_t length) {
	return eph_writepkt(iob,PKT_CMD,SEQ_INITCMD,data,length);
}

int eph_setispeed(eph_iob *iob,long val) {
	unsigned char buf[6];
	int rc;
	int count=0;

	buf[0]=CMD_SETINT;
	buf[1]=REG_SPEED;
	buf[2]=(val)&0xff;
	buf[3]=(val>>8)&0xff;
	buf[4]=(val>>16)&0xff;
	buf[5]=(val>>24)&0xff;
	do {
		if ((rc=eph_writeicmd(iob,buf,6))) return rc;
		rc=eph_waitack(iob,ACKTIMEOUT);
	} while (rc && (count++ < RETRIES));
	return rc;
}

int eph_setint(eph_iob *iob,int reg,long val) {
	unsigned char buf[6];
	int rc;
	int count=0;

	buf[0]=CMD_SETINT;
	buf[1]=reg;
	buf[2]=(val)&0xff;
	buf[3]=(val>>8)&0xff;
	buf[4]=(val>>16)&0xff;
	buf[5]=(val>>24)&0xff;

writeagain:
	if ((rc=eph_writecmd(iob,buf,6))) return rc;
	rc=eph_waitack(iob,(reg == REG_FRAME)?BIGACKTIMEOUT:ACKTIMEOUT);
	if (MAYRETRY(rc) && (count++ < RETRIES)) goto writeagain;

	return rc;
}

int eph_getint(eph_iob *iob,int reg,long *val) {
	unsigned char buf[4];
	eph_pkthdr pkt;
	int rc,size=4;
	int count=0;

	(*val)=0L;
	buf[0]=CMD_GETINT;
	buf[1]=reg;

writeagain:
	if ((rc=eph_writecmd(iob,buf,2))) return rc;
readagain:
	rc=eph_readpkt(iob,&pkt,buf,&size,BIGDATATIMEOUT);
	if (MAYRETRY(rc) && (count++ < RETRIES)) goto writeagain;
	if ((rc == 0) && (pkt.typ == PKT_LAST) && (pkt.seq == 0)) {
		(*val)=(buf[0]) | (buf[1]<<8) |
			(buf[2]<<16) | (buf[3]<<24);
		eph_writeack(iob);
		return 0;
	} else if ((rc == -1) && (count++ < RETRIES)) {
		eph_writenak(iob);
		goto readagain;
	}

	return rc;
}

int eph_action(eph_iob *iob,int reg,char *val,size_t length) {
	unsigned char buf[2050];
	int rc;
	int count=0;

	if (length > (sizeof(buf) - 2)) {
		eph_error(iob,ERR_DATA_TOO_LONG,"arg action length %ld",
				(long)length);
		return -1;
	}

	buf[0]=CMD_ACTION;
	buf[1]=reg;
	memcpy(buf+2,val,length);

writeagain:
	if ((rc=eph_writecmd(iob,buf,length+2))) return rc;
	rc=eph_waitack(iob,ACKTIMEOUT);
	if (MAYRETRY(rc) && (count++ < RETRIES)) goto writeagain;

	if (rc == 0) rc=eph_waitcomplete(iob);
	return rc;
}

int eph_setvar(eph_iob *iob,int reg,char *val,size_t length) {
	unsigned char buf[2050];
	int rc;
	int count=0;

	if (length > (sizeof(buf) - 2)) {
		eph_error(iob,ERR_DATA_TOO_LONG,"arg for setvar lenght %ld",
				(long)length);
		return -1;
	}

	buf[0]=CMD_SETVAR;
	buf[1]=reg;
	memcpy(buf+2,val,length);

writeagain:
	if ((rc=eph_writecmd(iob,buf,length+2))) return rc;
	rc=eph_waitack(iob,ACKTIMEOUT);
	if (MAYRETRY(rc) && (count++ < RETRIES)) goto writeagain;

	return rc;
}

int eph_getvar(eph_iob *iob,int reg,char **buffer,size_t *bufsize) {
	unsigned char buf[2];
	eph_pkthdr pkt;
	int rc;
	int count=0;
	unsigned char expect=0;
	size_t index,readsize;
	char *ptr;

	buf[0]=CMD_GETVAR;
	buf[1]=reg;

writeagain:
	if ((rc=eph_writecmd(iob,buf,2))) return rc;
	index=0;
readagain:
	if (((*bufsize) - index) < 2048) {
		if (iob->debug)
			printf("reallocing %lu",(unsigned long)(*bufsize));
		/* multiply current size by 2 and round up to 2048 boundary */
		(*bufsize)=((((*bufsize)*2)-1)/2048+1)*2048;
		if (iob->debug)
			printf(" -> %lu\n",(unsigned long)(*bufsize));
		(*buffer)=(iob->realloccb)(*buffer,*bufsize);
		if ((*buffer) == NULL) {
			eph_error(iob,ERR_NOMEM,
				"could not realloc %lu for getvar",
				(long)*bufsize);
			return -1;
		}
	}
	ptr=(*buffer)+index;
	readsize=(*bufsize)-index;
	rc=eph_readpkt(iob,&pkt,ptr,&readsize,
			(expect || ((reg != REG_IMG) || (reg != REG_TMN)))?
						DATATIMEOUT:BIGDATATIMEOUT);
	if (MAYRETRY(rc) && (expect == 0) && (count++ < RETRIES))
		goto writeagain;
	if ((rc == 0) &&
	    ((pkt.seq == expect) || (pkt.seq  == (expect-1)))) {
		count=0;
		if (pkt.seq == expect) {
			index+=readsize;
			expect++;
			(iob->runcb)(index);
		}
		eph_writeack(iob);
		if (pkt.typ == PKT_LAST) {
			(*bufsize)=index;
			return 0;
		}
		else goto readagain;
	}
	if ((rc <= 0) && (count++ < RETRIES)) {
		eph_writenak(iob);
		goto readagain;
	}
	return rc;
}
